/* ==========================================================================*/
/* - - - - - - - - - - -  P r o c D S P , C  - - - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*	Created   07-10-2004   M. Wheatley                                       */
/*	Modified  09-04-2004   MW                                                */
/*.........................................................................  */
/* DSP Processing Module                                                     */
/*.........................................................................  */

/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include <p30f6014.h>
#include <dsp.h>
#include "common.h"
#include "codec.h"
#include "menu.h"
#include "timers.h"
#include "generators.h"
#include "tables.h"

/*---------------------------------------------------------------------------*/
/*----------------> P R O C E S S O R    S P E C I F I C <-------------------*/
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*--------> T Y P E D E F S   U N I O N S    S T R U C T U R E S <-----------*/
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/
#define MAX_FIR 128
#define LMS_SIZE 24

/*---------------------------------------------------------------------------*/
/*----------------> E X T E R N A L  R E F E R E N C E S <-------------------*/
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*--------------> G L O B A L  M E M O R Y  A L L O C A T I O N <------------*/
/*---------------------------------------------------------------------------*/
unsigned int LMSmu;

/*---------------------------------------------------------------------------*/
/*--------------> L O C A L  M E M O R Y  A L L O C A T I O N <--------------*/
/*---------------------------------------------------------------------------*/
static fractional DelayBufI[MAX_FIR] __attribute__ ( (__section__(".ybss")) );
static fractional DelayBufQ[MAX_FIR] __attribute__ ( (__section__(".ybss")) );
static fractional LMSCoef[LMS_SIZE] __attribute__ ( (__section__(".xbss")) );
static FIRStruct FIRFilterI;
static FIRStruct FIRFilterQ;
static unsigned int page;
static fractional* offset;
static fractional pIBuf[PROC_BLOCK_SIZE];
static fractional pQBuf[PROC_BLOCK_SIZE];

static unsigned short isqrt(unsigned long a);

/*---------------------------------------------------------------------------*/
/*-----------------------> P R O T O T Y P E S <-----------------------------*/
/*---------------------------------------------------------------------------*/
void InitProcDSP( char DSPfunc );
void ProcLP( fractional* pIn, fractional* pOut);
void ProcLMS( fractional* pIn, fractional* pOut);
void ProcShift( fractional* pIn, fractional* pOut);

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                          -----------------------                          */
/*                         | I n i t P r o c D S P |                         */
/*                          -----------------------                          */
/*    This initializes the DSP Processing module.                            */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void InitProcDSP( char DSPfunc)
{
int i;
	switch(DSPfunc)
	{
		case DSP_NONE:
			break;
		case DSP_LPFIR:
			asm("mov #psvpage(_LP_1500),w0\nmov w0,_page");
			asm("mov #psvoffset(_LP_1500),w0\nmov w0,_offset");
			FIRStructInit(&FIRFilterI,LPFIR_SIZE, offset, page, DelayBufI);
		 	FIRDelayInit(&FIRFilterI);
			break;
		case DSP_HPFIR:
			asm("mov #psvpage(_HP_1500),w0\nmov w0,_page");
			asm("mov #psvoffset(_HP_1500),w0\nmov w0,_offset");
			FIRStructInit(&FIRFilterI,HPFIR_SIZE, offset, page, DelayBufI);
		 	FIRDelayInit(&FIRFilterI);
			break;
		case DSP_BPFIR:
			asm("mov #psvpage(_BP_1800),w0\nmov w0,_page");
			asm("mov #psvoffset(_BP_1800),w0\nmov w0,_offset");
			FIRStructInit(&FIRFilterI,BPFIR_SIZE, offset, page, DelayBufI);
		 	FIRDelayInit(&FIRFilterI);
			break;
		case DSP_BSFIR:
			asm("mov #psvpage(_BS_1800),w0\nmov w0,_page");
			asm("mov #psvoffset(_BS_1800),w0\nmov w0,_offset");
			FIRStructInit(&FIRFilterI,BSFIR_SIZE, offset, page, DelayBufI);
		 	FIRDelayInit(&FIRFilterI);
			break;
		case DSP_SHIFT:
			asm("mov #psvpage(_HILBPFIRI),w0\nmov w0,_page");
			asm("mov #psvoffset(_HILBPFIRI),w0\nmov w0,_offset");
			FIRStructInit(&FIRFilterI,HILBPFIRI_SIZE, offset, page, DelayBufI);
		 	FIRDelayInit(&FIRFilterI);
			asm("mov #psvpage(_HILBPFIRQ),w0\nmov w0,_page");
			asm("mov #psvoffset(_HILBPFIRQ),w0\nmov w0,_offset");
			FIRStructInit(&FIRFilterQ,HILBPFIRQ_SIZE, offset, page, DelayBufQ);
		 	FIRDelayInit(&FIRFilterQ);
			InitGen();
			break;
		case DSP_LMS:
			FIRStructInit(&FIRFilterI,LMS_SIZE, LMSCoef, COEFFS_IN_DATA, DelayBufI);
			for(i=0; i<LMS_SIZE; i++)
				LMSCoef[i] = 0;
		 	FIRDelayInit(&FIRFilterI);
			LMSmu = 15;
			break;
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                          -------------------                              */
/*                         | P r o c S h i f t |                             */
/*                          -------------------                              */
/*    This processes a buffers worth of samples through the Frequency shifter*/
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void ProcShift( fractional* pIn, fractional* pOut)
{
	FIR( PROC_BLOCK_SIZE, pIBuf, pIn, &FIRFilterI);	//create I data
	FIR( PROC_BLOCK_SIZE, pQBuf, pIn, &FIRFilterQ);	//create Q data
	MakeSinCos(PROC_BLOCK_SIZE, pIn, pOut, 0x7FFF);
	VectorMultiply(PROC_BLOCK_SIZE, pIBuf, pIBuf, pIn);
	VectorMultiply(PROC_BLOCK_SIZE, pQBuf, pQBuf, pOut);
	VectorAdd(PROC_BLOCK_SIZE, pOut, pIBuf, pQBuf);		//usb
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                          ---------------                                  */
/*                         | P r o c F I R |                                 */
/*                          ---------------                                  */
/*    This processes a buffers worth of samples through the FIR filter.      */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void ProcFIR( fractional* pIn, fractional* pOut)
{
	FIR( PROC_BLOCK_SIZE, pOut, pIn, &FIRFilterI);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                          ---------------                                  */
/*                         | P r o c L M S |                                 */
/*                          ---------------                                  */
/*    This processes a buffers worth of samples through the LMS filter.      */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void ProcLMS( fractional* pIn, fractional* pOut)
{
	FIRLMS( PROC_BLOCK_SIZE, pOut, pIn, &FIRFilterI, pIBuf, LMSmu);
	VectorCopy(PROC_BLOCK_SIZE, pIBuf, pIn);	//create delayed copy of input
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                          ---------                                        */
/*                         | s q r t |                                       */
/*                          ---------                                        */
/*    Finds square root of unsigned 32 bit value.                            */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static unsigned short isqrt(unsigned long a)
{
int i;
unsigned long rem = 0;
unsigned long root = 0;
	for(i=0; i<16; i++)
	{
		root <<= 1;
		rem = ((rem << 2) + (a >> 30));
		a <<= 2;
		root ++;
		if(root <= rem)
		{
			rem -= root;
			root++;
		}
		else
			root--;
	}
	return (unsigned short)(root >> 1);
}
